//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdio.h>
#include <winbase.h>
#pragma hdrstop

#include "FsExpert.h"
#include "FsOptions.h"

#include "mwIDEStream.hpp"
//---------------------------------------------------------------------------
#pragma package(smart_init)

__fastcall TFsExpert::TFsExpert(void) : Exptintf::TIExpert()
{
  char Path[MAX_PATH];
  GetModuleFileName(HInstance, Path, MAX_PATH);
  DllPath = ExtractFilePath(Path);
  if (!FileExists(DllPath+"indent.exe"))
  {   // indent.exe not in correct path
    AnsiString Msg;
    Msg.sprintf("Indent.exe and Index.pro not found in directory:\n");
    Msg += (DllPath + "\nPlease copy them there.");
    MessageDlg(Msg.c_str(), mtInformation, TMsgDlgButtons() << mbOK, 0);
  }
  else
  {
    TIMainMenuIntf *MainMenu;
    TIMenuItemIntf *ReferenceMenuItem;
    TIMenuItemIntf *Menu;
          // init vars
    CppFormatMenuItem = NULL;

    if (ToolServices)
    {
      MainMenu = ToolServices->GetMainMenu();
      if (MainMenu)
      { // then begin { we've got the main menu! }
        ReferenceMenuItem = MainMenu->FindMenuItem("EditSelectAll");
        if (ReferenceMenuItem)
        {
          Menu = ReferenceMenuItem->GetParent();
          if (Menu)
          {
            TIMenuFlags MenuFlags;
            CppFormatMenuItem = Menu->InsertItem(ReferenceMenuItem->GetIndex()+3,
                                "C++ Format",
                                "IDECppFormatSelection","Format Selected Text as C++",
                                TextToShortCut("Ctrl+Shift+C"),0,0,
                                MenuFlags << mfEnabled<< mfVisible, CppFormatSelectionClick);
            ReferenceMenuItem = MainMenu->FindMenuItem("ToolsToolsItem");
            if (ReferenceMenuItem)  // found tools - configure tools menu
            {
              Menu = ReferenceMenuItem->GetParent();
              if (Menu)
              {
                  TIMenuFlags MenuFlags;
                  CppOptionsMenuItem = Menu->InsertItem(ReferenceMenuItem->GetIndex()+1,
                                    "C++ Format Options",
                                    "IDECppFormatOptions","Set options for C++ Formatter",
                                    0,0,0,
                                    MenuFlags << mfEnabled<< mfVisible, CppFormatOptionsClick);
              }
            }
          }
        }
      }
    }
  }
}

__fastcall TFsExpert::~TFsExpert()
{
  if (CppFormatMenuItem)
    CppFormatMenuItem->DestroyMenuItem();
  if (CppOptionsMenuItem)
    CppOptionsMenuItem->DestroyMenuItem();
}

void __stdcall TFsExpert::Execute(void)
{
}

void __fastcall TFsExpert::CppFormatOptionsClick(TIMenuItemIntf * Sender)
{
#if 1
  FsOptForm = new TFsOptForm(NULL);
  if (FsOptForm->ShowModal() == mrOk)
    ;
  delete FsOptForm;
#endif
}

void __fastcall TFsExpert::CppFormatSelectionClick(TIMenuItemIntf * Sender)
{
  char TmpPath[MAX_PATH];
  char TmpFile[MAX_PATH];
  char ExecCmd[MAX_PATH];
//  GetTempPath(MAX_PATH, TmpPath);                       // get temp path
  GetTempFileName(".", "Cpp", 0, TmpFile);

  sprintf(ExecCmd, "\"%sIndent\" %s", DllPath.c_str(), TmpFile);
//  MessageDlg(ExecCmd, mtInformation, TMsgDlgButtons() << mbOK, 0);
  TIDEStream *IDEStream = new TIDEStream;
//  IDEStream:= TIDEStream.Create;
  IDEStream->WriteToFile(TmpFile);
  // now run indent on this file before....

  STARTUPINFO StartupInfo;
  ZeroMemory(&StartupInfo, sizeof(STARTUPINFO));
  StartupInfo.cb = sizeof(STARTUPINFO);
  // we are going to set the HIDE-WINDOW flag
  StartupInfo.dwFlags |= STARTF_USESHOWWINDOW;
  StartupInfo.wShowWindow = SW_HIDE;

  PROCESS_INFORMATION ProcessInfo;
  if (CreateProcess(NULL,
                    ExecCmd,
                    NULL,
                    NULL,
                    TRUE,
                    NORMAL_PRIORITY_CLASS,
                    NULL,
                    NULL,
                    &StartupInfo,
                    &ProcessInfo))
  {

    DWORD dwExitStatus = STILL_ACTIVE;
    // loop until indent closes
    do
    {
      if (!GetExitCodeProcess(ProcessInfo.hProcess, &dwExitStatus))
        break;
      Application->ProcessMessages();
    } while (dwExitStatus == STILL_ACTIVE);

    CloseHandle(ProcessInfo.hProcess);
    CloseHandle(ProcessInfo.hThread);
  }
  else
    MessageDlg("Couldn't run Indent.exe", mtInformation, TMsgDlgButtons() << mbOK, 0);

  // loading modified text
  IDEStream->ReadFromFile(TmpFile);
  DeleteFile(TmpFile);
  delete IDEStream;
}

System::AnsiString __stdcall TFsExpert::GetIDString()
{
  return "FraserSoft.CppFormat";
}

System::AnsiString __stdcall TFsExpert::GetMenuText()
{
  return "";
}

System::AnsiString __stdcall TFsExpert::GetName()
{
  return "CppFormat";
}

Exptintf::TExpertState __stdcall TFsExpert::GetState(void)
{
  Exptintf::TExpertState tx;
  tx << esEnabled;
  return tx;
}

Exptintf::TExpertStyle __stdcall TFsExpert::GetStyle(void)
{
  return esAddIn;
}

System::AnsiString __stdcall TFsExpert::GetAuthor()
{
  return "FraserSoft";
}

System::AnsiString __stdcall TFsExpert::GetComment()
{
  return "";
}

System::AnsiString __stdcall TFsExpert::GetPage()
{
  return "";
}

HICON __stdcall TFsExpert::GetGlyph(void)
{
  return 0;
}

namespace Fsexpert
{
  void __fastcall PACKAGE Register()
  {
    RegisterLibraryExpert(new TFsExpert());
  }
}

